#pragma once

#include <string>

#include "../../MemoryID.h"


namespace armarx::armem::base::detail
{

    /**
     * @brief Base class of memory classes on different levels.
     */
    class MemoryItem
    {
    public:

        MemoryItem();
        MemoryItem(const MemoryID& id);

        MemoryItem(const MemoryItem& other) = default;
        MemoryItem(MemoryItem&& other) = default;
        MemoryItem& operator=(const MemoryItem& other) = default;
        MemoryItem& operator=(MemoryItem&& other) = default;

        virtual ~MemoryItem();


        //MemoryItem& operator=(const MemoryItem& other);


        inline MemoryID& id()
        {
            return _id;
        }
        inline const MemoryID& id() const
        {
            return _id;
        }


        // Introspection

        /// Get a string version of `*this`' key.
        virtual std::string getKeyString() const = 0;

        /// Get a readable name of this level for messages, errors etc.
        virtual std::string getLevelName() const = 0;


    protected:

        /// Copy `*this` to `other`.
        virtual void _copySelf(MemoryItem& other) const;


    protected:

        MemoryID _id;

    };

}
