#pragma once

#include <string>
#include <vector>
#include <map>
#include <memory>

#include <bsoncxx/json.hpp>
#include <mongocxx/client.hpp>
#include <mongocxx/stdx.hpp>
#include <mongocxx/uri.hpp>
#include <mongocxx/instance.hpp>
#include <bsoncxx/builder/stream/helpers.hpp>
#include <bsoncxx/builder/stream/document.hpp>
#include <bsoncxx/builder/stream/array.hpp>


namespace armarx::armem::ltm
{

    using bsoncxx::builder::stream::close_array;
    using bsoncxx::builder::stream::close_document;
    using bsoncxx::builder::stream::document;
    using bsoncxx::builder::stream::finalize;
    using bsoncxx::builder::stream::open_array;
    using bsoncxx::builder::stream::open_document;


    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class MongoDBConnectionManager
    {
    public:
        struct MongoDBSettings
        {
            std::string host = "localhost";
            unsigned int port = 27017;
            std::string user = "";
            std::string password = "";
            std::string database = "Test";


            bool isSet() const
            {
                // we always need a user and a host
                return !host.empty() and port != 0 and !user.empty();
            }

            std::string uri() const
            {
                return "mongodb://" + host + ":" + std::to_string(port) + user;
            }

            std::string toString() const
            {
                return uri() + ":" + password + "/" + database;
            }
        };

        static mongocxx::client& EstablishConnection(const MongoDBSettings& settings);


    private:
        static bool initialized;
        static std::map<std::string, mongocxx::client> Connections;

    };
}
