#pragma once

#include <RobotAPI/interface/armem/query.h>


namespace armarx::armem::base::query_proc
{

    /**
     * @brief Base class for memory query processors.
     */
    template <class DataT, class QueryT>
    class BaseQueryProcessorBase
    {
    public:
        using QueryPtrT = ::IceInternal::Handle<QueryT>;
        using QuerySeqT = std::vector<QueryPtrT>;

    public:

        DataT process(const QueryT& query, const DataT& data, const std::vector<query::data::QueryTarget>& ignoreTargets = {}) const
        {
            DataT result = data.copyEmpty();

            if (std::find(ignoreTargets.begin(), ignoreTargets.end(), query.target) != ignoreTargets.end())
            {
                return result;
            }
            this->process(result, query, data);
            return result;
        }

        DataT process(const QueryPtrT& query, const DataT& data, const std::vector<query::data::QueryTarget>& ignoreTargets = {}) const
        {
            return this->process(*query, *data, ignoreTargets);
        }

        DataT process(const QuerySeqT& queries, const DataT& data, const std::vector<query::data::QueryTarget>& ignoreTargets = {}) const
        {
            DataT result = data.copyEmpty();
            this->process(result, queries, data, ignoreTargets);
            return result;
        }

        void process(DataT& result, const QuerySeqT& queries, const DataT& data, const std::vector<query::data::QueryTarget>& ignoreTargets = {}) const
        {
            for (const auto& query : queries)
            {
                if (std::find(ignoreTargets.begin(), ignoreTargets.end(), query->target) != ignoreTargets.end())
                {
                    continue;
                }
                this->process(result, *query, data);
            }
        }

        virtual void process(DataT& result, const QueryT& query, const DataT& data) const = 0;
    };
}
