#pragma once

#include <map>

#include <QWidget>

#include <RobotAPI/libraries/armem/core/DataMode.h>
#include <RobotAPI/libraries/armem/client/query/Builder.h>

#include "SnapshotForm.h"


class QCheckBox;
class QComboBox;
class QVBoxLayout;


namespace armarx::armem::gui
{

    /**
     * @brief Widget where queries for snapshots can be configured.
     *
     * Manages multiple forms which are hidden if not selected
     * (in order to keep their settings).
     */
    class SnapshotSelectorWidget : public QWidget
    {
        Q_OBJECT
        using This = SnapshotSelectorWidget;

    public:

        SnapshotSelectorWidget();


        armem::DataMode dataMode() const;
        client::query::SnapshotSelector& selector();
        query::data::QueryTarget queryTarget() const;


    public slots:

    signals:

        void queryChanged();
        void queryTargetChanged();


    private slots:

        void updateSelector();
        void updateQueryTarget();

        void hideAllForms();
        void showSelectedFormForQuery(QString selected);


    signals:

        void queryOutdated();
        void queryTargetOutdated();


    private:

        void addForm(const QString& key, SnapshotForm* form);


    public:

        client::query::SnapshotSelector _selector;
        query::data::QueryTarget _queryTarget;

        QVBoxLayout* _pageLayout;
        QComboBox* _queryComboBox;
        QComboBox* _queryTargetComboBox;
        /// The forms for the different query types. Hidden when not selected.
        std::map<QString, SnapshotForm*> _queryForms;
        std::map<QString, query::data::QueryTarget> _queryTargets;

    };

}
