/*
* This file is part of ArmarX.
*
* Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
* Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <vector>
#include <map>
#include <string>

// Simox
#include <SimoxUtility/algorithm/string.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/navigator/Navigator.h>

namespace armarx::aron::typenavigator
{
    class NavigatorFactory;
    typedef std::shared_ptr<NavigatorFactory> NavigatorFactoryPtr;

    class Navigator;
    typedef std::shared_ptr<Navigator> NavigatorPtr;

    class Navigator :
        virtual public aron::Navigator<type::Descriptor, type::AronType>
    {
    public:
        using PointerType = NavigatorPtr;

    public:
        // constructors
        Navigator() = default;

        // operators
        virtual bool operator==(const Navigator& other) const = 0;
        virtual bool operator==(const NavigatorPtr& other) const = 0;

        // static methods
        static NavigatorPtr FromAronType(const type::AronTypePtr&, const Path& = Path());
        static std::vector<NavigatorPtr> FromAronType(const std::vector<type::AronTypePtr>&, const Path& = Path());
        static std::vector<type::AronTypePtr> FromAronType(const std::vector<NavigatorPtr>&);

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCast(const NavigatorPtr& n)
        {
            return NavigatorT::DynamicCast(n);
        }

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCast(const Navigator& n)
        {
            return NavigatorT::DynamicCast(n);
        }

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCastAndCheck(const NavigatorPtr& n)
        {
            return NavigatorT::DynamicCastAndCheck(n);
        }

        // virtual methods
        virtual std::vector<NavigatorPtr> getChildren() const = 0;
        virtual size_t childrenSize() const = 0;
        virtual void setMaybe(const type::Maybe m) = 0;
        virtual type::Maybe getMaybe() const = 0;

        // public member functions
        type::AronTypePtr toAronTypePtr() const
        {
            return toAronPtr();
        }

    private:
        static const NavigatorFactoryPtr FACTORY;
    };
}
