/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <vector>

// Base Class
#include "../detail/ContainerNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class ListNavigator;
    typedef std::shared_ptr<ListNavigator> ListNavigatorPtr;

    class ListNavigator :
            virtual public detail::ContainerNavigatorBase<type::AronList, ListNavigator>
    {
    public:
        // constructors
        ListNavigator(const Path& path = Path());
        ListNavigator(const type::AronListPtr&, const Path& path = Path());

        // operators
        virtual bool operator==(const ListNavigator&) const override;

        // public member functions
        NavigatorPtr getAcceptedType() const;
        void setAcceptedType(const NavigatorPtr&);

        type::AronListPtr toAronListPtr() const;

        // virtual implementations
        std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

    private:
        // members
        NavigatorPtr acceptedType;
    };
}
