/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include "../detail/ContainerNavigatorBase.h"


namespace armarx::aron::typenavigator
{
    class ObjectNavigator;
    typedef std::shared_ptr<ObjectNavigator> ObjectNavigatorPtr;

    class ObjectNavigator :
            virtual public detail::ContainerNavigatorBase<type::AronObject, ObjectNavigator>
    {
    public:
        // constructors
        ObjectNavigator(const Path& = Path());
        ObjectNavigator(const std::string&, const Path& = Path());
        ObjectNavigator(const type::AronObjectPtr&, const Path& = Path());

        // operators
        virtual bool operator==(const ObjectNavigator&) const override;

        // public member functions
        bool checkObjectName(const std::string&) const;

        std::map<std::string, NavigatorPtr> getMemberTypes() const;
        NavigatorPtr getMemberType(const std::string&) const;
        std::string getObjectName() const;
        ObjectNavigatorPtr getExtends() const;

        void setObjectName(const std::string&);
        void setExtends(const ObjectNavigatorPtr&);
        void addMemberType(const std::string&, const NavigatorPtr&);

        bool hasMemberType(const std::string&) const;

        std::vector<std::string> getAllKeys() const;

        type::AronObjectPtr toAronObjectPtr() const;

        // virtual implementations
        std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

    private:
        // members
        ObjectNavigatorPtr extends;
        std::map<std::string, NavigatorPtr> memberTypes;
    };
}
