/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NDArrayNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class NDArrayNavigator;
    typedef std::shared_ptr<NDArrayNavigator> NDArrayNavigatorPtr;

    class NDArrayNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronNDArray, NDArrayNavigator>
    {
    public:
        // constructors
        NDArrayNavigator(const Path& path = Path());
        NDArrayNavigator(const type::AronNDArrayPtr&, const Path& path = Path());

        // operators
        virtual bool operator==(const NDArrayNavigator&) const override;

        // public member functions
        std::string getTypename() const;
        std::vector<int> getDimensions() const;
        unsigned int getElementSize() const;

        void setTypename(const std::string&);
        void setDimensions(const std::vector<int>&);
        void addDimension(int);
        void setElementSize(unsigned int);

        type::AronNDArrayPtr toAronNDArrayPtr() const;

        // virtual implementations
        virtual std::string getName() const override;
    };
}
